"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProgressBar = void 0;
var ansi_js_1 = __importDefault(require("ansi.js"));
var get_cursor_position_1 = __importDefault(require("get-cursor-position"));
var constants_1 = require("./constants");
var global_1 = require("./global");
var util = __importStar(require("./util"));
var ProgressBar = /** @class */ (function () {
    function ProgressBar(options) {
        if (options === void 0) { options = {}; }
        this.cursor = (0, ansi_js_1.default)(constants_1.stream);
        this.total = options.total || 100;
        this.current = options.current || 0;
        var width = options.width || 60;
        if (typeof width === 'string') {
            if (width.endsWith('%')) {
                this.width = Math.round(parseFloat(width) / 100);
            }
            else {
                this.width = parseFloat(width);
            }
        }
        else {
            this.width = width;
        }
        this.fixedWidth = options.fixedWidth === true;
        this.tough = options.tough === true;
        this.clean = options.clean === true;
        this.blankChar = options.blank || '—';
        this.fillChar = options.filled || '▇';
        if (options.callback) {
            this.callback = options.callback;
        }
        this.setSchema(options.schema);
        this.snoop();
        global_1.instances.push(this);
    }
    ProgressBar.prototype.setSchema = function (schema, refresh) {
        this.schema = schema || ' [:bar] :current/:total :percent :elapseds :etas';
        if (refresh) {
            this.compile(refresh);
        }
    };
    ProgressBar.prototype.tick = function (delta, tokens) {
        if (typeof delta === 'object') {
            // eslint-disable-next-line no-param-reassign
            tokens = delta;
            // eslint-disable-next-line no-param-reassign
            delta = 1;
        }
        else if (typeof delta === 'undefined') {
            // eslint-disable-next-line no-param-reassign
            delta = 1;
        }
        else {
            // eslint-disable-next-line no-param-reassign
            delta = parseFloat(delta);
            if (Number.isNaN(delta) || !Number.isFinite(delta)) {
                // eslint-disable-next-line no-param-reassign
                delta = 1;
            }
        }
        if (this.completed && delta >= 0) {
            return this;
        }
        if (!this.start) {
            this.start = +new Date();
        }
        this.current += delta;
        this.compile(tokens);
        this.snoop();
        return this;
    };
    ProgressBar.prototype.update = function (ratio, tokens) {
        var completed = Math.floor(ratio * this.total);
        var delta = completed - this.current;
        return this.tick(delta, tokens);
    };
    ProgressBar.prototype.clear = function () {
        if (this.output) {
            this.cursor.moveTo(this.originPos.row, this.originPos.col);
            for (var i = 0; i < this.rows; i++) {
                this.cursor.eraseLine().moveDown();
            }
            this.cursor.moveTo(this.originPos.row, this.originPos.col);
        }
        return this;
    };
    ProgressBar.prototype.compile = function (tokens) {
        var ratio = this.current / this.total;
        ratio = Math.min(Math.max(ratio, 0), 1);
        var schema = this.schema;
        var percent = ratio * 100;
        var elapsed = +new Date() - this.start;
        var eta = this.current <= 0
            ? '-'
            : util.formatTime(percent === 100 ? 0 : (elapsed * this.total) / this.current);
        var output = schema
            .replace(/:total/g, "".concat(this.total))
            .replace(/:current/g, "".concat(this.current))
            .replace(/:elapsed/g, util.formatTime(elapsed))
            .replace(/:eta/g, eta)
            .replace(/:percent/g, "".concat(util.toFixed(percent, 0), "%"));
        if (tokens && typeof tokens === 'object') {
            Object.keys(tokens).forEach(function (key) {
                output = output.replace(new RegExp(":".concat(key), 'g'), "".concat(tokens[key]) || constants_1.placeholder);
            });
        }
        var raw = util.bleach(output);
        var cols = process.stdout.columns;
        var width = this.width;
        width = width < 1 ? cols * width : width;
        if (!this.fixedWidth) {
            width = Math.min(width, Math.max(0, cols - util.bareLength(raw)));
        }
        var length = Math.round(width * ratio);
        var filled = util.repeatChar(this.fillChar, length);
        var blank = util.repeatChar(this.blankChar, width - length);
        raw = util.combine(raw, filled, blank, true);
        output = util.combine(output, filled, blank, false);
        // without color and font styles
        this.raw = raw;
        // row count of the progress bar
        this.rows = raw.split('\n').length;
        this.render(output);
    };
    ProgressBar.prototype.render = function (output) {
        var _this = this;
        if (this.output === output) {
            return;
        }
        var current = get_cursor_position_1.default.sync();
        if (!current) {
            return;
        }
        (0, global_1.beginUpdate)();
        this.savedPos = current;
        if (!this.originPos) {
            this.originPos = current;
        }
        if (this.originPos.row === constants_1.stream.rows) {
            this.cursor.write(util.repeatChar('\n', this.rows));
            global_1.instances.forEach(function (instance) {
                if (instance.originPos) {
                    instance.originPos.row -= _this.rows;
                }
            });
        }
        this.clear();
        this.colorize(output);
        // move the cursor to the current position.
        if (this.rendered) {
            this.cursor.moveTo(current.row, current.col);
        }
        this.output = output;
        this.rendered = true;
        (0, global_1.endUpdate)();
    };
    ProgressBar.prototype.colorize = function (output) {
        var cursor = this.cursor;
        var parts = output.split(/(\.[A-Za-z]+)/g);
        var charsLeft = process.stdout.columns - 1;
        var content = '';
        var matches = [];
        cursor.moveTo(this.originPos.row, this.originPos.col);
        var write = function () {
            var hasFg = false;
            var hasBg = false;
            var gradient;
            matches.forEach(function (match) {
                if (match.method === 'gradient') {
                    gradient = match;
                    return;
                }
                var host = match.isBg
                    ? cursor.background
                    : match.isFont
                        ? cursor.font
                        : cursor.foreground;
                if (match.isBg) {
                    hasBg = true;
                }
                else {
                    hasFg = true;
                }
                host[match.method]();
            });
            content = content.replace(new RegExp(constants_1.placeholder, 'g'), '');
            if (content) {
                if (gradient) {
                    var color1 = gradient.color1;
                    var color2 = gradient.color2;
                    for (var i = 0, l = content.length; i < l; i++) {
                        var color = i === 0
                            ? color1
                            : i === l - 1
                                ? color2
                                : util.interpolate(color1, color2, (i + 1) / l);
                        cursor.foreground.rgb(color.r, color.g, color.b);
                        writeChars(content[i]);
                        cursor.foreground.reset();
                    }
                }
                else {
                    writeChars(content);
                }
            }
            // reset font style
            matches.forEach(function (match) {
                if (match.isFont) {
                    var method = "reset".concat(util.ucFirst(match.method));
                    var font = cursor.font;
                    font[method]();
                }
            });
            if (hasFg) {
                // reset foreground
                cursor.foreground.reset();
            }
            if (hasBg) {
                // reset background
                cursor.background.reset();
            }
            matches = [];
            content = '';
        };
        var writeChars = function (chars) {
            if (!charsLeft) {
                return;
            }
            cursor.write(chars.slice(0, charsLeft));
            charsLeft = Math.max(0, charsLeft - chars.length);
        };
        for (var i = 0, l = parts.length; i < l; i++) {
            var part = parts[i];
            var match = void 0;
            if (!part) {
                continue;
            }
            if (part.startsWith('.')) {
                if (part === '.gradient') {
                    if (parts[i + 1]) {
                        match = util.parseGradient(parts[i + 1]);
                        parts[i + 1] = parts[i + 1].replace(/^\((.+),(.+)\)/, '');
                    }
                }
                else {
                    match = util.parseMethod(cursor, part);
                }
            }
            if (match) {
                var suffix = match.suffix;
                if (suffix) {
                    if (i < l - 1) {
                        parts[i + 1] += suffix;
                    }
                    else {
                        // the last one
                        writeChars(suffix);
                    }
                }
                matches.push(match);
            }
            else {
                if (matches.length) {
                    write();
                }
                content += part;
            }
        }
        write();
        cursor.write('\n');
    };
    ProgressBar.prototype.snoop = function () {
        this.completed = this.current >= this.total;
        if (this.completed) {
            this.terminate();
        }
        return this.completed;
    };
    ProgressBar.prototype.terminate = function () {
        if (this.clean && this.rendered) {
            this.clear();
        }
        if (this.callback) {
            this.callback(this);
        }
        if (this.savedPos) {
            this.cursor.moveTo(this.savedPos.row, this.savedPos.col);
        }
    };
    return ProgressBar;
}());
exports.ProgressBar = ProgressBar;
//# sourceMappingURL=progress-bar.js.map